## INTRODUCTION

The Media From Images module automatically creates media entities for image files that do not already have corresponding media entities. This module helps maintain consistency between file entities and media entities in your Drupal site, ensuring that all image files are properly represented as media entities for better content management.

The module provides a comprehensive tracking system using SHA256 file hashes to prevent duplicate media creation and maintain data integrity.

The primary use cases for this module are:

- Migrating from a site where files exist but media entities were not created
- Ensuring all uploaded image files have corresponding media entities
- Maintaining data consistency between file and media systems
- Batch processing large numbers of existing image files
- Tracking relationships between files and media entities using cryptographic hashes

## FEATURES

- **Automatic Media Creation**: Creates media entities for image files without existing media representations
- **Hash-Based Tracking**: Uses SHA256 file hashes to track media-file relationships and prevent duplicates
- **Batch Processing**: Processes large numbers of files in manageable batches with progress tracking
- **Operation Locking**: Prevents concurrent operations that could cause data conflicts
- **Existing Media Attachment**: Identifies and tracks existing media entities not yet in the tracking system
- **Administrative Interface**: Web-based interface showing statistics and allowing batch operations
- **Drush Integration**: Command-line tools for server-side processing
- **Automatic Cleanup**: Removes tracking entries when media or file entities are deleted
- **Comprehensive Logging**: Detailed logging of all operations for monitoring and debugging

## REQUIREMENTS

This module requires the following modules:

- **Media** (Drupal core)
- **File** (Drupal core)

## INSTALLATION

Install as you would normally install a contributed Drupal module. Visit
https://www.drupal.org/node/1897420 for further information.

### Steps:
1. Copy the module to your `modules/custom` directory
2. Enable the module through the Drupal administrative interface or using Drush:
   ```
   drush en media_from_images
   ```
3. The module will automatically create the required database table upon installation
4. Navigate to Configuration > Media > Media From Images to access the administrative interface

## CONFIGURATION

The module works with minimal configuration:

1. **Ensure Image Media Bundle**: Verify that an 'image' media bundle exists in your site
2. **Check Field Configuration**: The image media bundle should have a 'field_media_image' field for file references
3. **Permissions**: Users need 'administer site configuration' permission to access the administrative interface
4. **File System**: Ensure image files are managed through Drupal's file system

No additional configuration steps are required. The module uses sensible defaults and integrates with existing Drupal media and file systems.

## USAGE

### Administrative Interface

Navigate to **Configuration > Media > Media From Images** (`/admin/config/media/media-from-images`) to:

- View statistics about image files and media entities
- See current tracking table entries with links to media and file entities
- Start batch processes to create media entities for untracked files
- Attach existing media entities to the tracking system
- Monitor operation status and prevent concurrent operations

### Drush Commands

Process files via command line:

```bash
# Create media entities for all image files that don't have them
drush media-from-images:process
# or use the alias
drush mfi:process
```

### API Usage

The module provides a service for programmatic access:

```php
$service = \Drupal::service('media_from_images.service');

// Get all image file IDs
$file_ids = $service->getImageFileIds();

// Check if media entity exists for a file
$file = File::load($file_id);
$exists = $service->mediaEntityExistsForFile($file);

// Create media entity from file
$service->createMediaFromFile($file);

// Start batch process
$service->startBatchProcess();
```

## DATABASE SCHEMA

The module creates a custom table `media_from_images` with:

- **media_image_hash** (varchar 255, primary key): SHA256 hash of the image file
- **media_image_id** (int 10): Media entity ID
- **image_entity_id** (int 10): File entity ID

## TROUBLESHOOTING

### Common Issues

- **Button doesn't work on first click**: This has been resolved in the current version
- **Batch operation appears stuck**: Operation locks expire automatically after 30 minutes
- **"Field field_media_image not found"**: Ensure your image media bundle has the correct field name
- **File not found errors**: Verify image files physically exist on the filesystem
- **Permission denied**: Ensure users have 'administer site configuration' permission

### Debugging

Check the logs at **Reports > Recent log messages** for detailed error information. All operations are logged with the 'media_from_images' channel.

## UNINSTALLATION

When uninstalling the module:

1. Disable the module: `drush pmu media_from_images`
2. The custom database table will be automatically removed
3. Media entities created by this module will remain in your system

## TECHNICAL NOTES

- Uses Drupal's batch API for processing large numbers of files
- Implements operation locking using Drupal's lock service
- Automatically cleans up tracking entries via entity hooks
- Processes files in batches of 50 (configurable in code)
- Includes comprehensive error handling and logging
- File hashes are generated using SHA256 for reliable duplicate detection

## MAINTAINERS

Current maintainers for Drupal 10+:

- Module development and maintenance ongoing

