<?php

namespace Drupal\media_from_images\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\media_from_images\MediaFromImagesServiceInterface;
use Drush\Commands\DrushCommands;

/**
 * Drush commands for Media From Images module.
 */
class MediaFromImagesCommands extends DrushCommands {

  /**
   * The media from images service.
   *
   * @var \Drupal\media_from_images\MediaFromImagesServiceInterface
   */
  protected MediaFromImagesServiceInterface $mediaFromImagesService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a MediaFromImagesCommands object.
   *
   * @param \Drupal\media_from_images\MediaFromImagesServiceInterface $media_from_images_service
   *   The media from images service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    MediaFromImagesServiceInterface $media_from_images_service,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    parent::__construct();
    $this->mediaFromImagesService = $media_from_images_service;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Create media entities from all image files that don't have them.
   *
   * @command media-from-images:process
   * @aliases mfi:process
   */
  public function processImageFiles(): void {
    $this->output()->writeln('Starting batch process to create media entities from image files...');

    $file_ids = $this->mediaFromImagesService->getImageFileIds();

    if (empty($file_ids)) {
      $this->output()->writeln('No image files found to process.');
      return;
    }

    $this->output()->writeln(sprintf('Found %d image files to process.', count($file_ids)));

    $processed = 0;
    $created = 0;
    $skipped = 0;

    $file_storage = $this->entityTypeManager->getStorage('file');

    foreach ($file_ids as $file_id) {
      /** @var \Drupal\file\FileInterface $file */
      $file = $file_storage->load($file_id);

      if (!$file) {
        $skipped++;
        continue;
      }

      if (!$this->mediaFromImagesService->mediaEntityExistsForFile($file)) {
        $this->mediaFromImagesService->createMediaFromFile($file);
        $created++;
        $this->output()->writeln(sprintf('Created media entity for file ID %d', $file_id));
      }
      else {
        $skipped++;
      }

      $processed++;

      // Show progress every 50 files.
      if ($processed % 50 === 0) {
        $this->output()->writeln(sprintf('Processed %d of %d files...', $processed, count($file_ids)));
      }
    }

    $this->output()->writeln(sprintf(
      'Process completed. Processed: %d, Created: %d, Skipped: %d',
      $processed,
      $created,
      $skipped
    ));
  }

}