<?php

namespace Drupal\media_from_images;

use Drupal\Core\Database\Connection;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;

/**
 * Service for creating media entities from image files.
 */
class MediaFromImagesService implements MediaFromImagesServiceInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   *
   * @todo Fix problem: Implement logic to generate file hash.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The lock backend service.
   *
   * @var \Drupal\Core\Lock\LockBackendInterface
   */
  protected LockBackendInterface $lock;

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * The time service.
   *
   * @var \Drupal\Core\Datetime\TimeInterface
   */
  protected TimeInterface $time;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * Constructs a MediaFromImagesService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Lock\LockBackendInterface $lock
   *   The lock backend service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user account.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelInterface $logger,
    FileSystemInterface $file_system,
    Connection $database,
    LockBackendInterface $lock,
    AccountProxyInterface $current_user,
    StateInterface $state,
    TimeInterface $time,
    MessengerInterface $messenger,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
    $this->fileSystem = $file_system;
    $this->database = $database;
    $this->lock = $lock;
    $this->currentUser = $current_user;
    $this->state = $state;
    $this->time = $time;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public function getImageFileIds(): array {
    $file_storage = $this->entityTypeManager->getStorage('file');

    $query = $file_storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('filemime', 'image/%', 'LIKE');

    $file_ids = $query->execute();

    return array_values($file_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function getFileHash(FileInterface $file): ?string {
    $file_uri = $file->getFileUri();
    $file_path = $this->fileSystem->realpath($file_uri);

    // Security: Validate file path and ensure it's within expected directories.
    if (!$file_path || !file_exists($file_path)) {
      $this->logger->error('File does not exist: @uri (ID: @id)', [
        '@uri' => $file_uri,
        '@id' => $file->id(),
      ]);
      return NULL;
    }

    // Security: Ensure the file is readable and is actually an image.
    if (!is_readable($file_path)) {
      $this->logger->error('File is not readable: @path (ID: @id)', [
        '@path' => $file_path,
        '@id' => $file->id(),
      ]);
      return NULL;
    }

    // Security: Validate that the file is actually an image by checking MIME.
    $mime_type = $file->getMimeType();
    if (!str_starts_with($mime_type, 'image/')) {
      $this->logger->error('File is not an image: @mime (ID: @id)', [
        '@mime' => $mime_type,
        '@id' => $file->id(),
      ]);
      return NULL;
    }

    return hash_file('sha256', $file_path);
  }

  /**
   * {@inheritdoc}
   */
  public function mediaEntityExistsForFile(FileInterface $file): bool {
    $hash = $this->getFileHash($file);

    if (!$hash) {
      return FALSE;
    }

    // First check the custom tracking table.
    $query = $this->database->select('media_from_images', 'mfi')
      ->fields('mfi', ['media_image_hash'])
      ->condition('media_image_hash', $hash)
      ->range(0, 1);

    $result = $query->execute()->fetchField();

    if (!empty($result)) {
      return TRUE;
    }

    // If not found in tracking table, check for existing media entities.
    $media_storage = $this->entityTypeManager->getStorage('media');
    $media_query = $media_storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('bundle', 'image')
      ->condition('field_media_image', $file->id())
      ->range(0, 1);

    $media_ids = $media_query->execute();

    return !empty($media_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function createMediaFromFile(FileInterface $file): void {
    // Security: Validate that this is actually an image file.
    $mime_type = $file->getMimeType();
    if (strpos($mime_type, 'image/') !== 0) {
      $this->logger->error('Attempted to create media entity for non-image file: @mime (ID: @id)', [
        '@mime' => $mime_type,
        '@id' => $file->id(),
      ]);
      return;
    }

    try {
      $media_storage = $this->entityTypeManager->getStorage('media');

      // Security: Sanitize filename for use as media name.
      $safe_filename = preg_replace('/[^a-zA-Z0-9._-]/', '_', $file->getFilename());

      $media = $media_storage->create([
        'bundle' => 'image',
        'name' => $safe_filename,
        'field_media_image' => [
          'target_id' => $file->id(),
          'alt' => $safe_filename,
        ],
      ]);

      // Save the media entity first to get an ID.
      $media->save();

      // Create hash entry in tracking table after saving.
      $this->createMediaHashEntry($media, $file);

      $this->logger->info('Created media entity @media_id for file @file_id', [
        '@media_id' => $media->id(),
        '@file_id' => $file->id(),
      ]);
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to create media from file @file_id: @message', [
        '@file_id' => $file->id(),
        '@message' => $e->getMessage(),
      ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createMediaHashEntry(MediaInterface $media, FileInterface $file): bool {
    $hash = $this->getFileHash($file);

    if (!$hash) {
      $this->logger->error('Cannot create hash entry - unable to generate hash for file ID: @file_id', [
        '@file_id' => $file->id(),
      ]);
      return FALSE;
    }

    try {
      $this->database->insert('media_from_images')
        ->fields([
          'media_image_hash' => $hash,
          'media_image_id' => $media->id(),
          'image_entity_id' => $file->id(),
        ])
        ->execute();

      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to create media hash entry: @message', [
        '@message' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function startBatchProcess(): void {
    $lock_name = 'media_from_images_create_batch';
    $rate_limit_key = 'media_from_images_rate_limit_' . $this->currentUser->id();

    // Check rate limiting - allow one operation per user per 5 minutes.
    $last_operation = $this->state->get($rate_limit_key, 0);
    $current_time = $this->time->getRequestTime();
    if (($current_time - $last_operation) < 300) {
      $this->logger->warning('Rate limit exceeded for user @uid attempting batch process.', [
        '@uid' => $this->currentUser->id(),
      ]);
      $this->messenger->addWarning($this->t('Please wait at least 5 minutes between batch operations.'));
      return;
    }

    // Try to acquire lock with 30 minute timeout.
    if (!$this->lock->acquire($lock_name, 1800)) {
      $this->logger->warning('Cannot start batch process - another operation is already running.');
      $this->messenger->addWarning($this->t('Another batch operation is currently running. Please wait for it to complete.'));
      return;
    }

    // Set rate limit timestamp.
    $this->state->set($rate_limit_key, $current_time);

    $file_ids = $this->getImageFileIds();

    if (empty($file_ids)) {
      $this->lock->release($lock_name);
      $this->logger->info('No image files found to process.');
      return;
    }

    $operations = [];
    $batch_size = 50;

    // Split file IDs into batches.
    $batches = array_chunk($file_ids, $batch_size);

    foreach ($batches as $batch_file_ids) {
      $operations[] = [
        '\Drupal\media_from_images\MediaFromImagesService::processBatch',
        [$batch_file_ids],
      ];
    }

    $batch = [
      'title' => $this->t('Creating media entities from image files'),
      'operations' => $operations,
      'finished' => '\Drupal\media_from_images\MediaFromImagesService::batchFinished',
      'progress_message' => $this->t('Processing @current of @total batches.'),
    ];

    batch_set($batch);
  }

  /**
   * Batch process callback to create media entities.
   *
   * @param array $file_ids
   *   Array of file IDs to process.
   * @param array $context
   *   Batch context array.
   */
  public static function processBatch(array $file_ids, array &$context) {
    $service = \Drupal::service('media_from_images.service');
    $entity_type_manager = \Drupal::entityTypeManager();
    $file_storage = $entity_type_manager->getStorage('file');

    if (!isset($context['sandbox']['processed'])) {
      $context['sandbox']['processed'] = 0;
      $context['sandbox']['created'] = 0;
      $context['sandbox']['skipped'] = 0;
    }

    foreach ($file_ids as $file_id) {
      $file = $file_storage->load($file_id);

      if (!$file) {
        $context['sandbox']['skipped']++;
        continue;
      }

      if (!$service->mediaEntityExistsForFile($file)) {
        $service->createMediaFromFile($file);
        $context['sandbox']['created']++;
      }
      else {
        $context['sandbox']['skipped']++;
      }

      $context['sandbox']['processed']++;
    }

    $context['message'] = t('Processed @processed files. Created @created media entities. Skipped @skipped files.', [
      '@processed' => $context['sandbox']['processed'],
      '@created' => $context['sandbox']['created'],
      '@skipped' => $context['sandbox']['skipped'],
    ]);

    $context['results']['processed'] = $context['sandbox']['processed'];
    $context['results']['created'] = $context['sandbox']['created'];
    $context['results']['skipped'] = $context['sandbox']['skipped'];
  }

  /**
   * Batch finished callback.
   *
   * @param bool $success
   *   Indicates whether the batch process was successful.
   * @param array $results
   *   Array of results from the batch process.
   * @param array $operations
   *   Array of operations that were processed.
   */
  public static function batchFinished($success, array $results, array $operations) {
    $messenger = \Drupal::messenger();
    $lock = \Drupal::service('lock');

    // Release the lock.
    $lock->release('media_from_images_create_batch');

    if ($success) {
      $message = t('Batch process completed successfully. Processed @processed files, created @created media entities, skipped @skipped files.', [
        '@processed' => $results['processed'] ?? 0,
        '@created' => $results['created'] ?? 0,
        '@skipped' => $results['skipped'] ?? 0,
      ]);
      $messenger->addStatus($message);
    }
    else {
      $messenger->addError(t('Batch process finished with errors.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getMediaHashEntries(int $limit = 50, int $offset = 0): array {
    $query = $this->database->select('media_from_images', 'mfi')
      ->fields('mfi', ['media_image_hash', 'media_image_id', 'image_entity_id'])
      ->orderBy('media_image_id', 'DESC')
      ->range($offset, $limit);

    $results = $query->execute()->fetchAll();

    $entries = [];
    $media_storage = $this->entityTypeManager->getStorage('media');
    $file_storage = $this->entityTypeManager->getStorage('file');

    foreach ($results as $result) {
      $media = $media_storage->load($result->media_image_id);
      $file = $file_storage->load($result->image_entity_id);

      $entries[] = [
        'hash' => $result->media_image_hash,
        'media_id' => $result->media_image_id,
        'media_name' => $media ? $media->getName() : 'Deleted',
        'media_link' => $media ? $media->toUrl('edit-form') : NULL,
        'file_id' => $result->image_entity_id,
        'filename' => $file ? $file->getFilename() : 'Deleted',
        'file_link' => $file ? $file->createFileUrl() : NULL,
        'filesize' => $file ? $this->formatBytes($file->getSize()) : 'N/A',
      ];
    }

    return $entries;
  }

  /**
   * {@inheritdoc}
   */
  public function countMediaHashEntries(): int {
    $query = $this->database->select('media_from_images', 'mfi')
      ->countQuery();

    return (int) $query->execute()->fetchField();
  }

  /**
   * {@inheritdoc}
   */
  public function countFilesWithoutMediaEntities(): int {
    $file_ids = $this->getImageFileIds();
    $file_storage = $this->entityTypeManager->getStorage('file');

    $count = 0;
    foreach ($file_ids as $file_id) {
      $file = $file_storage->load($file_id);
      if ($file && !$this->mediaEntityExistsForFile($file)) {
        $count++;
      }
    }

    return $count;
  }

  /**
   * {@inheritdoc}
   */
  public function countUnattachedMediaEntities(): int {
    $media_storage = $this->entityTypeManager->getStorage('media');
    $media_query = $media_storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('bundle', 'image');

    $all_media_ids = $media_query->execute();

    // Get media IDs that are already tracked.
    $tracked_query = $this->database->select('media_from_images', 'mfi')
      ->fields('mfi', ['media_image_id']);

    $tracked_media_ids = $tracked_query->execute()->fetchCol();

    // Count untracked media entities.
    $untracked_media_ids = array_diff($all_media_ids, $tracked_media_ids);

    return count($untracked_media_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function startAttachUnattachedProcess(): void {
    $lock_name = 'media_from_images_attach_batch';
    $rate_limit_key = 'media_from_images_attach_rate_limit_' . $this->currentUser->id();

    // Check rate limiting - allow one operation per user per 5 minutes.
    $last_operation = $this->state->get($rate_limit_key, 0);
    $current_time = $this->time->getRequestTime();
    if (($current_time - $last_operation) < 300) {
      $this->logger->warning('Rate limit exceeded for user @uid attempting attach process.', [
        '@uid' => $this->currentUser->id(),
      ]);
      $this->messenger->addWarning($this->t('Please wait at least 5 minutes between batch operations.'));
      return;
    }

    // Try to acquire lock with 30 minute timeout.
    if (!$this->lock->acquire($lock_name, 1800)) {
      $this->logger->warning('Cannot start attach process - another operation is already running.');
      $this->messenger->addWarning($this->t('Another batch operation is currently running. Please wait for it to complete.'));
      return;
    }

    // Set rate limit timestamp.
    $this->state->set($rate_limit_key, $current_time);

    $media_storage = $this->entityTypeManager->getStorage('media');
    $media_query = $media_storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('bundle', 'image');

    $all_media_ids = $media_query->execute();

    // Get media IDs that are already tracked.
    $tracked_query = $this->database->select('media_from_images', 'mfi')
      ->fields('mfi', ['media_image_id']);

    $tracked_media_ids = $tracked_query->execute()->fetchCol();

    // Get untracked media IDs.
    $untracked_media_ids = array_diff($all_media_ids, $tracked_media_ids);

    if (empty($untracked_media_ids)) {
      $this->lock->release($lock_name);
      $this->logger->info('No unattached media entities found to process.');
      return;
    }

    $operations = [];
    $batch_size = 50;

    // Split media IDs into batches.
    $batches = array_chunk($untracked_media_ids, $batch_size);

    foreach ($batches as $batch_media_ids) {
      $operations[] = [
        '\Drupal\media_from_images\MediaFromImagesService::processAttachBatch',
        [$batch_media_ids],
      ];
    }

    $batch = [
      'title' => $this->t('Attaching unattached media entities to hash table'),
      'operations' => $operations,
      'finished' => '\Drupal\media_from_images\MediaFromImagesService::attachBatchFinished',
      'progress_message' => $this->t('Processing @current of @total batches.'),
    ];

    batch_set($batch);
  }

  /**
   * Batch process callback to attach unattached media entities.
   *
   * @param array $media_ids
   *   Array of media IDs to process.
   * @param array $context
   *   Batch context array.
   */
  public static function processAttachBatch(array $media_ids, array &$context) {
    $service = \Drupal::service('media_from_images.service');
    $entity_type_manager = \Drupal::entityTypeManager();
    $media_storage = $entity_type_manager->getStorage('media');
    $file_storage = $entity_type_manager->getStorage('file');

    if (!isset($context['sandbox']['processed'])) {
      $context['sandbox']['processed'] = 0;
      $context['sandbox']['attached'] = 0;
      $context['sandbox']['skipped'] = 0;
    }

    foreach ($media_ids as $media_id) {
      $media = $media_storage->load($media_id);

      if (!$media) {
        $context['sandbox']['skipped']++;
        continue;
      }

      // Get the file from the media entity.
      $file_field = $media->get('field_media_image');
      if ($file_field->isEmpty()) {
        $context['sandbox']['skipped']++;
        continue;
      }

      $file_id = $file_field->target_id;
      $file = $file_storage->load($file_id);

      if (!$file) {
        $context['sandbox']['skipped']++;
        continue;
      }

      // Create hash entry for this media/file pair.
      if ($service->createMediaHashEntry($media, $file)) {
        $context['sandbox']['attached']++;
      }
      else {
        $context['sandbox']['skipped']++;
      }

      $context['sandbox']['processed']++;
    }

    $context['message'] = t('Processed @processed media entities. Attached @attached. Skipped @skipped.', [
      '@processed' => $context['sandbox']['processed'],
      '@attached' => $context['sandbox']['attached'],
      '@skipped' => $context['sandbox']['skipped'],
    ]);

    $context['results']['processed'] = $context['sandbox']['processed'];
    $context['results']['attached'] = $context['sandbox']['attached'];
    $context['results']['skipped'] = $context['sandbox']['skipped'];
  }

  /**
   * Batch finished callback for attach process.
   *
   * @param bool $success
   *   Indicates whether the batch process was successful.
   * @param array $results
   *   Array of results from the batch process.
   * @param array $operations
   *   Array of operations that were processed.
   */
  public static function attachBatchFinished($success, array $results, array $operations) {
    $messenger = \Drupal::messenger();
    $lock = \Drupal::service('lock');

    // Release the lock.
    $lock->release('media_from_images_attach_batch');

    if ($success) {
      $message = t('Attach process completed successfully. Processed @processed media entities, attached @attached, skipped @skipped.', [
        '@processed' => $results['processed'] ?? 0,
        '@attached' => $results['attached'] ?? 0,
        '@skipped' => $results['skipped'] ?? 0,
      ]);
      $messenger->addStatus($message);
    }
    else {
      $messenger->addError(t('Attach process finished with errors.'));
    }
  }

  /**
   * Format bytes into human readable format.
   *
   * @param int $bytes
   *   The number of bytes.
   * @param int $precision
   *   The number of decimal places.
   *
   * @return string
   *   The formatted string.
   */
  protected function formatBytes($bytes, $precision = 2) {
    $units = ['B', 'KB', 'MB', 'GB', 'TB'];

    for ($i = 0; $bytes > 1024 && $i < count($units) - 1; $i++) {
      $bytes /= 1024;
    }

    return round($bytes, $precision) . ' ' . $units[$i];
  }

  /**
   * {@inheritdoc}
   */
  public function isBatchOperationRunning(): bool {
    return $this->lock->lockMayBeAvailable('media_from_images_create_batch') === FALSE ||
           $this->lock->lockMayBeAvailable('media_from_images_attach_batch') === FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function deleteHashEntryByMediaId(int $media_id): bool {
    try {
      $deleted = $this->database->delete('media_from_images')
        ->condition('media_image_id', $media_id)
        ->execute();

      if ($deleted > 0) {
        $this->logger->info('Deleted hash table entry for media ID @media_id', [
          '@media_id' => $media_id,
        ]);
        return TRUE;
      }

      return FALSE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to delete hash table entry for media ID @media_id: @message', [
        '@media_id' => $media_id,
        '@message' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteHashEntryByFileId(int $file_id): bool {
    try {
      $deleted = $this->database->delete('media_from_images')
        ->condition('image_entity_id', $file_id)
        ->execute();

      if ($deleted > 0) {
        $this->logger->info('Deleted hash table entry for file ID @file_id', [
          '@file_id' => $file_id,
        ]);
        return TRUE;
      }

      return FALSE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to delete hash table entry for file ID @file_id: @message', [
        '@file_id' => $file_id,
        '@message' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

}
