<?php

namespace Drupal\media_from_images;

use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;

/**
 * Interface for the Media from Images service.
 */
interface MediaFromImagesServiceInterface {

  /**
   * Get image file IDs.
   *
   * @return array
   *   Array of image file IDs.
   */
  public function getImageFileIds(): array;

  /**
   * Get file hash.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   *
   * @return string|null
   *   The file hash or NULL if unable to generate.
   */
  public function getFileHash(FileInterface $file): ?string;

  /**
   * Check if media entity exists for file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   *
   * @return bool
   *   TRUE if media entity exists, FALSE otherwise.
   */
  public function mediaEntityExistsForFile(FileInterface $file): bool;

  /**
   * Create media from file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   */
  public function createMediaFromFile(FileInterface $file): void;

  /**
   * Create media hash entry in tracking table.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media entity.
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   *
   * @return bool
   *   TRUE if entry was created successfully, FALSE otherwise.
   */
  public function createMediaHashEntry(MediaInterface $media, FileInterface $file): bool;

  /**
   * Start batch process to create media entities for all image files.
   */
  public function startBatchProcess(): void;

  /**
   * Get all media hash entries for display.
   *
   * @param int $limit
   *   Number of entries to return. Defaults to 50.
   * @param int $offset
   *   Offset for pagination. Defaults to 0.
   *
   * @return array
   *   Array of media hash entries with media and file information.
   */
  public function getMediaHashEntries(int $limit = 50, int $offset = 0): array;

  /**
   * Count total media hash entries.
   *
   * @return int
   *   Total number of media hash entries.
   */
  public function countMediaHashEntries(): int;

  /**
   * Count files without media entities.
   *
   * @return int
   *   Number of image files without corresponding media entities.
   */
  public function countFilesWithoutMediaEntities(): int;

  /**
   * Count unattached media entities (media entities not in hash table).
   *
   * @return int
   *   Number of media entities that exist but are not tracked in hash table.
   */
  public function countUnattachedMediaEntities(): int;

  /**
   * Start batch process to attach existing media entities to hash table.
   */
  public function startAttachUnattachedProcess(): void;

  /**
   * Check if a batch operation is currently running.
   *
   * @return bool
   *   TRUE if a batch operation is running, FALSE otherwise.
   */
  public function isBatchOperationRunning(): bool;

  /**
   * Delete hash table entry by media ID.
   *
   * @param int $media_id
   *   The media entity ID.
   *
   * @return bool
   *   TRUE if entry was deleted, FALSE otherwise.
   */
  public function deleteHashEntryByMediaId(int $media_id): bool;

  /**
   * Delete hash table entry by file ID.
   *
   * @param int $file_id
   *   The file entity ID.
   *
   * @return bool
   *   TRUE if entry was deleted, FALSE otherwise.
   */
  public function deleteHashEntryByFileId(int $file_id): bool;

}
