<?php

namespace Drupal\Tests\media_from_images\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;

/**
 * Tests access control for Media From Images module.
 *
 * @group media_from_images
 */
class MediaFromImagesAccessTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'media_from_images',
    'media',
    'file',
    'image',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installSchema('system', ['sequences']);
  }

  /**
   * Tests that the custom permission is properly defined.
   */
  public function testPermissionDefinition(): void {
    $permissions = \Drupal::service('user.permissions')->getPermissions();

    $this->assertArrayHasKey('administer media from images', $permissions);

    $permission = $permissions['administer media from images'];
    $this->assertEquals('Administer Media From Images', $permission['title']);
    $this->assertEquals('Access the Media From Images administration pages and manage batch operations.', $permission['description']);
    $this->assertTrue($permission['restrict access']);
  }

  /**
   * Tests permission assignment and checking.
   */
  public function testPermissionAssignment(): void {
    // Create a role with the permission.
    $role = Role::create([
      'id' => 'media_admin',
      'label' => 'Media Administrator',
    ]);
    $role->grantPermission('administer media from images');
    $role->save();

    // Create a user with this role.
    $user = User::create([
      'name' => 'media_admin_user',
      'mail' => 'admin@example.com',
      'status' => 1,
    ]);
    $user->addRole('media_admin');
    $user->save();

    // Test that the user has the permission.
    $this->assertTrue($user->hasPermission('administer media from images'));

    // Create a user without the permission.
    $regular_user = User::create([
      'name' => 'regular_user',
      'mail' => 'user@example.com',
      'status' => 1,
    ]);
    $regular_user->save();

    // Test that the user does not have the permission.
    $this->assertFalse($regular_user->hasPermission('administer media from images'));
  }

  /**
   * Tests that the permission properly restricts access.
   */
  public function testPermissionRestriction(): void {
    // Test that the permission has the 'restrict access' flag.
    $permissions = \Drupal::service('user.permissions')->getPermissions();
    $permission = $permissions['administer media from images'];

    $this->assertTrue($permission['restrict access'], 'Permission should have restrict access flag set.');
  }

  /**
   * Tests service access through permission system.
   */
  public function testServicePermissionIntegration(): void {
    // Create a user with the permission.
    $admin_user = User::create([
      'name' => 'admin',
      'mail' => 'admin@example.com',
      'status' => 1,
    ]);
    $admin_user->save();

    $role = Role::create([
      'id' => 'admin_role',
      'label' => 'Admin Role',
    ]);
    $role->grantPermission('administer media from images');
    $role->save();

    $admin_user->addRole('admin_role');
    $admin_user->save();

    // Switch to this user.
    \Drupal::currentUser()->setAccount($admin_user);

    // Verify the service is accessible.
    $service = \Drupal::service('media_from_images.service');
    $this->assertNotNull($service);
  }

}
