<?php

namespace Drupal\Tests\media_from_images\Kernel;

use Drupal\file\Entity\File;
use Drupal\KernelTests\KernelTestBase;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Tests input validation and security for Media From Images module.
 *
 * @group media_from_images
 */
class MediaFromImagesValidationTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'media_from_images',
    'media',
    'file',
    'image',
    'field',
  ];

  /**
   * The media from images service.
   *
   * @var \Drupal\media_from_images\MediaFromImagesServiceInterface
   */
  protected $service;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('file');
    $this->installEntitySchema('media');
    $this->installSchema('file', ['file_usage']);
    $this->installSchema('system', ['sequences']);
    $this->installSchema('media_from_images', ['media_from_images']);

    // Create image media type.
    $media_type = MediaType::create([
      'id' => 'image',
      'label' => 'Image',
      'source' => 'image',
    ]);
    $media_type->save();

    // Set the source field for the media type.
    $media_type->set('source_configuration', [
      'source_field' => 'field_media_image',
    ]);
    $media_type->save();

    // Create field_media_image field.
    FieldStorageConfig::create([
      'field_name' => 'field_media_image',
      'entity_type' => 'media',
      'type' => 'image',
    ])->save();

    FieldConfig::create([
      'field_name' => 'field_media_image',
      'entity_type' => 'media',
      'bundle' => 'image',
      'label' => 'Image',
    ])->save();

    $this->service = \Drupal::service('media_from_images.service');
  }

  /**
   * Tests file type validation.
   */
  public function testFileTypeValidation(): void {
    // Create a non-image file.
    $non_image_file = File::create([
      'filename' => 'document.pdf',
      'uri' => 'public://document.pdf',
      'filemime' => 'application/pdf',
      'status' => 1,
    ]);
    $non_image_file->save();

    // Test that getFileHash rejects non-image files.
    $hash = $this->service->getFileHash($non_image_file);
    $this->assertNull($hash, 'Non-image files should not generate hashes.');

    // Create a proper image file.
    $image_file = File::create([
      'filename' => 'test.jpg',
      'uri' => 'public://test.jpg',
      'filemime' => 'image/jpeg',
      'status' => 1,
    ]);
    $image_file->save();

    // This should work (though will fail due to missing physical file).
    $hash = $this->service->getFileHash($image_file);
    // We expect null because the physical file doesn't exist, but the validation should pass.
    $this->assertNull($hash, 'Hash should be null when physical file is missing.');
  }

  /**
   * Tests filename sanitization.
   */
  public function testFilenameSanitization(): void {
    // Create a file with malicious filename.
    $malicious_file = File::create([
      'filename' => '<script>alert("XSS")</script>.jpg',
      'uri' => 'public://malicious.jpg',
      'filemime' => 'image/jpeg',
      'status' => 1,
    ]);
    $malicious_file->save();

    // Create a mock physical file to avoid file existence errors.
    file_put_contents('public://malicious.jpg', 'fake image content');

    // The service should sanitize the filename when creating media.
    $this->service->createMediaFromFile($malicious_file);

    // Check that any created media entities have sanitized names.
    $media_entities = \Drupal::entityTypeManager()
      ->getStorage('media')
      ->loadByProperties(['bundle' => 'image']);

    foreach ($media_entities as $media) {
      $name = $media->getName();
      $this->assertStringNotContainsString('<script>', $name, 'Media name should not contain script tags.');
      $this->assertStringNotContainsString('</script>', $name, 'Media name should not contain script tags.');
      $this->assertStringNotContainsString('<', $name, 'Media name should not contain HTML tags.');
      $this->assertStringNotContainsString('>', $name, 'Media name should not contain HTML tags.');
      // Verify the filename has been sanitized to replace dangerous characters.
      $this->assertMatchesRegularExpression('/^[a-zA-Z0-9._-]+$/', $name, 'Media name should only contain safe characters.');
    }
  }

  /**
   * Tests that database operations are secure.
   */
  public function testDatabaseSecurity(): void {
    // Test that the service properly handles special characters in hashes.
    $special_hash = "'; DROP TABLE media_from_images; --";

    // Create mock media and file entities.
    $file = File::create([
      'filename' => 'test.jpg',
      'uri' => 'public://test.jpg',
      'filemime' => 'image/jpeg',
      'status' => 1,
    ]);
    $file->save();

    $media = Media::create([
      'bundle' => 'image',
      'name' => 'Test Image',
      'field_media_image' => ['target_id' => $file->id()],
    ]);
    $media->save();

    // Attempt to delete with a malicious ID (this should be safe due to parameterized queries).
    $result = $this->service->deleteHashEntryByMediaId($media->id());

    // The table should still exist and be intact.
    $database = \Drupal::database();
    $table_exists = $database->schema()->tableExists('media_from_images');
    $this->assertTrue($table_exists, 'Database table should still exist after operations.');
  }

  /**
   * Tests rate limiting enforcement.
   */
  public function testRateLimitingValidation(): void {
    // Create a mock current user.
    $account = $this->createMock('\Drupal\Core\Session\AccountInterface');
    $account->method('id')->willReturn(1);

    \Drupal::currentUser()->setAccount($account);

    // Set a recent operation timestamp.
    $current_time = \Drupal::time()->getRequestTime();
    \Drupal::state()->set('media_from_images_rate_limit_1', $current_time);

    // Mock the lock service to allow the operation.
    $lock = $this->createMock('\Drupal\Core\Lock\LockBackendInterface');
    $lock->method('acquire')->willReturn(TRUE);

    // The service should respect rate limiting.
    $this->service->startBatchProcess();

    // Check that appropriate message was set (this would be tested in functional tests).
    // For unit tests, we can verify the state was checked.
    $last_operation = \Drupal::state()->get('media_from_images_rate_limit_1');
    $this->assertEquals($current_time, $last_operation);
  }

  /**
   * Tests that media entities are properly validated before creation.
   */
  public function testMediaEntityValidation(): void {
    // Create a file that looks like an image but isn't.
    $fake_image = File::create([
      'filename' => 'fake.jpg',
      'uri' => 'public://fake.jpg',
    // Wrong MIME type.
      'filemime' => 'text/plain',
      'status' => 1,
    ]);
    $fake_image->save();

    // The service should reject this file.
    $this->service->createMediaFromFile($fake_image);

    // Verify no media entities were created.
    $media_entities = \Drupal::entityTypeManager()
      ->getStorage('media')
      ->loadByProperties(['bundle' => 'image']);

    $this->assertEmpty($media_entities, 'No media entities should be created for non-image files.');
  }

}
