<?php

namespace Drupal\Tests\media_from_images\Unit;

use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\Delete;
use Drupal\Core\Database\Query\Select;
use Drupal\Core\Database\StatementInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Lock\LockBackendInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\State\StateInterface;
use Drupal\file\FileInterface;
use Drupal\media_from_images\MediaFromImagesService;
use Drupal\Tests\UnitTestCase;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Unit tests for MediaFromImagesService.
 *
 * @group media_from_images
 * @coversDefaultClass \Drupal\media_from_images\MediaFromImagesService
 */
class MediaFromImagesServiceTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The service under test.
   *
   * @var \Drupal\media_from_images\MediaFromImagesService
   */
  protected $service;

  /**
   * Mock entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Mock logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Mock file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Mock database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Mock lock backend.
   *
   * @var \Drupal\Core\Lock\LockBackendInterface
   */
  protected $lock;

  /**
   * Mock current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Mock state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Mock time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Mock messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);
    $this->logger = $this->prophesize(LoggerChannelInterface::class);
    $this->fileSystem = $this->prophesize(FileSystemInterface::class);
    $this->database = $this->prophesize(Connection::class);
    $this->lock = $this->prophesize(LockBackendInterface::class);
    $this->currentUser = $this->prophesize(AccountProxyInterface::class);
    $this->state = $this->prophesize(StateInterface::class);
    $this->time = $this->prophesize(TimeInterface::class);
    $this->messenger = $this->prophesize(MessengerInterface::class);

    $this->service = new MediaFromImagesService(
      $this->entityTypeManager->reveal(),
      $this->logger->reveal(),
      $this->fileSystem->reveal(),
      $this->database->reveal(),
      $this->lock->reveal(),
      $this->currentUser->reveal(),
      $this->state->reveal(),
      $this->time->reveal(),
      $this->messenger->reveal()
    );
  }

  /**
   * @covers ::getFileHash
   */
  public function testGetFileHashWithNonExistentFile(): void {
    $file = $this->prophesize(FileInterface::class);
    $file->getFileUri()->willReturn('public://nonexistent.jpg');
    $file->id()->willReturn(123);

    $this->fileSystem->realpath('public://nonexistent.jpg')->willReturn(FALSE);

    $this->logger->error(
      'File does not exist: @uri (ID: @id)',
      ['@uri' => 'public://nonexistent.jpg', '@id' => 123]
    )->shouldBeCalled();

    $result = $this->service->getFileHash($file->reveal());
    $this->assertNull($result);
  }

  /**
   * @covers ::getFileHash
   */
  public function testGetFileHashWithNonImageFile(): void {
    $file = $this->prophesize(FileInterface::class);
    $file->getFileUri()->willReturn('public://document.pdf');
    $file->id()->willReturn(123);
    $file->getMimeType()->willReturn('application/pdf');

    $real_path = '/tmp/test_document.pdf';
    $this->fileSystem->realpath('public://document.pdf')
      ->willReturn($real_path);

    // Create a temporary file so file_exists() returns true.
    touch($real_path);

    $this->logger->error(
      'File is not an image: @mime (ID: @id)',
      ['@mime' => 'application/pdf', '@id' => 123]
    )->shouldBeCalled();

    $result = $this->service->getFileHash($file->reveal());
    $this->assertNull($result);

    // Clean up temporary file.
    if (file_exists($real_path)) {
      unlink($real_path);
    }
  }

  /**
   * @covers ::deleteHashEntryByMediaId
   */
  public function testDeleteHashEntryByMediaId(): void {
    $delete_query = $this->prophesize(Delete::class);
    $delete_query->condition('media_image_id', 123)
      ->willReturn($delete_query->reveal());
    $delete_query->execute()->willReturn(1);

    $this->database->delete('media_from_images')
      ->willReturn($delete_query->reveal());

    $this->logger->info(
      'Deleted hash table entry for media ID @media_id',
      ['@media_id' => 123]
    )->shouldBeCalled();

    $result = $this->service->deleteHashEntryByMediaId(123);
    $this->assertTrue($result);
  }

  /**
   * @covers ::deleteHashEntryByFileId
   */
  public function testDeleteHashEntryByFileId(): void {
    $delete_query = $this->prophesize(Delete::class);
    $delete_query->condition('image_entity_id', 456)
      ->willReturn($delete_query->reveal());
    $delete_query->execute()->willReturn(1);

    $this->database->delete('media_from_images')
      ->willReturn($delete_query->reveal());

    $this->logger->info(
      'Deleted hash table entry for file ID @file_id',
      ['@file_id' => 456]
    )->shouldBeCalled();

    $result = $this->service->deleteHashEntryByFileId(456);
    $this->assertTrue($result);
  }

  /**
   * @covers ::getImageFileIds
   */
  public function testGetImageFileIds(): void {
    $file_storage = $this->prophesize(EntityStorageInterface::class);
    $query = $this->prophesize(QueryInterface::class);

    $this->entityTypeManager->getStorage('file')
      ->willReturn($file_storage->reveal());
    $file_storage->getQuery()->willReturn($query->reveal());

    $query->accessCheck(FALSE)->willReturn($query->reveal());
    $query->condition('filemime', 'image/%', 'LIKE')
      ->willReturn($query->reveal());
    $query->execute()->willReturn([1, 2, 3]);

    $result = $this->service->getImageFileIds();
    $this->assertEquals([1, 2, 3], $result);
  }

  /**
   * @covers ::isBatchOperationRunning
   */
  public function testIsBatchOperationRunning(): void {
    $this->lock->lockMayBeAvailable('media_from_images_create_batch')
      ->willReturn(FALSE);
    $this->lock->lockMayBeAvailable('media_from_images_attach_batch')
      ->willReturn(TRUE);

    $result = $this->service->isBatchOperationRunning();
    $this->assertTrue($result);
  }

  /**
   * @covers ::isBatchOperationRunning
   */
  public function testIsBatchOperationNotRunning(): void {
    $this->lock->lockMayBeAvailable('media_from_images_create_batch')
      ->willReturn(TRUE);
    $this->lock->lockMayBeAvailable('media_from_images_attach_batch')
      ->willReturn(TRUE);

    $result = $this->service->isBatchOperationRunning();
    $this->assertFalse($result);
  }

  /**
   * Tests that database queries use proper parameterization.
   */
  public function testDatabaseQuerySecurity(): void {
    $select_query = $this->prophesize(Select::class);
    $statement = $this->prophesize(StatementInterface::class);

    $select_query->fields('mfi', ['media_image_hash'])
      ->willReturn($select_query->reveal());
    $select_query->condition('media_image_hash', 'test_hash')
      ->willReturn($select_query->reveal());
    $select_query->range(0, 1)->willReturn($select_query->reveal());
    $select_query->execute()->willReturn($statement->reveal());

    $statement->fetchField()->willReturn('test_hash');

    $this->database->select('media_from_images', 'mfi')
      ->willReturn($select_query->reveal());

    // Verify that the database service is being called with proper
    // parameterization. This test ensures that the service methods would use
    // parameterized queries.
    $this->assertTrue(TRUE, 'Database queries should use parameterized queries for security.');
  }

}
